(function($) {

	var debounce = function(func, wait, immediate) {
		var timeout;
		return function() {
		    var context = this, args = arguments;
		    var later = function() {
		        timeout = null;
		        if (!immediate) func.apply(context, args);
		    };
		    var callNow = immediate && !timeout;
		    clearTimeout(timeout);
		    timeout = setTimeout(later, wait);
		    if (callNow) func.apply(context, args);
		};
	};

	var validate = function(fields, mandatory) {

		/**
		 * Get required fileds that are missing
		 */
		var required = mandatory.filter(function(key) {
			return !fields[key];
		});

		return required;
	}

	var handleFavorResponse = function(userResponse) {

		var data = {
			action: 'emojics_favor_response',
			response: userResponse
		};

		var req = $.ajax({
			url: window.emojics_data.ajax_url,
			method: 'POST',
			data: data,
			dataType: 'json'
		});

		req.done(function(res) {
			if ( res.success ) {
				$('.emojics-favor-message').slideUp('fast');
				setTimeout(function() {
					window.dispatchEvent(new Event('resize'));
				}, 500);
			}
		});
	}

	var initDashboard = function() {

		/**
		 * Elements
		 */
		var $frame = $('.emojics-frame-wrap__frame');
		var $form = $('#emojics-frame-login');

		/**
		 * Functions
		 */
		var resizeFrame = function() {
			var framePosition = $frame.position();
			$frame.height($(window).height() - framePosition.top - 50);
		}

		/**
		 * Listeners
		 */
		window.addEventListener('resize', debounce(resizeFrame, 200, false), false);

		resizeFrame();
    	$form.submit();
	    $frame.on('load', function() {
	    	$form.remove();
	    });
	}

	$(document).ready(function() {

		$('.emojics-authenticate').on('submit', function(e) {
			e.preventDefault();
			var $el_submit = $(this).find('button.authenticate');
			var $el_error = $(this).find('.emojics-error-message');
			var submit_text = $el_submit.text();

			var data = {
				'action': 'emojics_api_authentication',
				'emojics_api_key': $('#emojics_api_key').val()
			};

			/**
			 * Get required fields that are missing
			 */
			var required = validate(data, ['emojics_api_key']);

			if ( required.length ) {
				$el_error.text('API Key is required');
				return false;
			}

			$el_submit.text('Loading...');
			$el_error.text('');
			var req = $.ajax({
				url: $(this).attr('action'),
				method: 'POST',
				data: data,
				dataType: 'json'
			});

			req.done(function(res) {
				$el_submit.text(submit_text);

				if ( res.success ) {
					location.reload();
				}
				else {
					$el_error.text('Something is wrong with the API Key you provided');
				}
			});
		});

		$('.emojics-registration').on('submit', function(e) {
			e.preventDefault();

			var $el_submit = $(this).find('button.register');
			var $el_error = $(this).find('.emojics-error-message');
			var submit_text = $el_submit.text();

			var data = {
				'action': 'emojics_api_registration',
				'first_name': $('#emojics_first_name').val(),
				'last_name': $('#emojics_last_name').val(),
				'site': $('#emojics_site').val(),
				'email': $('#emojics_email').val(),
				'password': $('#emojics_password').val()
			};

			/**
			 * Get required fields that are missing
			 */
			var required = validate(data, ['site', 'email', 'password']);

			if ( required.length ) {
				$el_error.text('There are some mandatory fields that are missing');
				return false;
			}

			$el_submit.text('Loading...');
			$el_error.text('');
			$('.emojics-field').removeClass('error');
			var req = $.ajax({
				url: $(this).attr('action'),
				method: 'POST',
				data: data,
				dataType: 'json'
			});

			req.done(function(res) {
				$el_submit.text(submit_text);

				if ( res.success ) {
					// alert('registration successful');
					location.reload();
				}
				else {

					if ( res.error ) {
						$el_error.text(res.error);
					}
					else {
						$el_error.text('Something went wrong with the sign up. Try manual registration at app.emojics.com');
					}

					if ( res.errors ) {
						Object.keys(res.errors).map(function(key) {
							$('#emojics_' + key).addClass('error');
							$el_error.html( $el_error.html() + '<br /> - ' + res.errors[key] );

						});
					}

				}
			});
		});

		$('.emojics-switch-signup-signin').on('click', function() {

			if ( $('#emojics-registration').hasClass('active') ) {
				$('#emojics-registration').removeClass('active');
				$('#emojics-authenticate').addClass('active');
			}
			else {
				$('#emojics-authenticate').removeClass('active');
				$('#emojics-registration').addClass('active');
			}
		});

		$('.emojics-widget-enabled').on('change', function() {

			var $other_options = $(this).parents('.form-table').next();
			if ( $(this).val() === 'yes' ) {
				$other_options.show();
			}
			else {
				$other_options.hide();
			}
		});

		if ( $('#emojics-dashboard').length ) {
			initDashboard();
		}

		$('.emojics-favor-message__ok').on('click', function() {
			handleFavorResponse('ok');
		});

		$('.emojics-favor-message__later').on('click', function() {
			handleFavorResponse('later');
		});

		$('.emojics-favor-message__already').on('click', function() {
			handleFavorResponse('already');
		});

		$('.emojics-favor-message__close').on('click', function() {
			handleFavorResponse('after_2_days');
		});
	});
})(jQuery)
